@@ -19,12 +19,11 @@ use constant _DUMP_TOKEN => scalar($DEBUG =~ /\b dump=token \b/xmsi);
 
 our @CARP_NOT = qw(Text::Xslate::Compiler Text::Xslate::Symbol);
 
-my $CODE    = qr/ (?: (?: $STRING | [^'"] )*? ) /xms; # ' for poor editors
-
+my $CODE    = qr/ (?: $STRING | [^'"] ) /xms;
 my $COMMENT = qr/\# [^\n;]* (?=[;\n])?/xms;
 
 # Operator tokens that the parser recognizes.
-# All the single characters are tokanized as an operator.
+# All the single characters are tokenized as an operator.
 my $OPERATOR_TOKEN = sprintf '(?:%s|[^ \t\r\n])', join('|', map{ quotemeta } qw(
     ...
     ..
@@ -208,7 +207,7 @@ sub parse {
 
     $parser->input( $parser->preprocess($input) );
 
-    $parser->next_token( $parser->tokanize() );
+    $parser->next_token( $parser->tokenize() );
     $parser->advance();
     my $ast = $parser->statements();
 
@@ -278,7 +277,7 @@ sub auto_chomp {
     return $nl;
 }
 
-# split templates by tags before tokanizing
+# split templates by tags before tokenizing
 sub split :method {
     my $parser  = shift;
     local($_) = @_;
@@ -291,7 +290,6 @@ sub split :method {
 
     my $lex_line_code = defined($line_start) && qr/\A ^ [ \t]* \Q$line_start\E ([^\n]* \n?) /xms;
     my $lex_tag_start = qr/\A \Q$tag_start\E ($CHOMP_FLAGS?)/xms;
-    my $lex_tag_end   = qr/\A ($CODE) ($CHOMP_FLAGS?) \Q$tag_end\E/xms;
 
     my $lex_text = qr/\A ( [^\n]*? (?: \n | (?= \Q$tag_start\E ) | \z ) ) /xms;
 
@@ -299,10 +297,25 @@ sub split :method {
 
     while($_) {
         if($in_tag) {
-            if(s/$lex_tag_end//xms) {
-                my($code, $chomp) = ($1, $2);
-                $in_tag = 0;
+            my $start = 0;
+            my $pos;
+            while( ($pos = index $_, $tag_end, $start) >= 0 ) {
+                my $code = substr $_, 0, $pos;
+                $code =~ s/$CODE//xmsog;
+                if(length($code) == 0) {
+                    last;
+                }
+                $start = $pos + 1;
+            }
 
+            if($pos >= 0) {
+                my $code = substr $_, 0, $pos, '';
+                $code =~ s/($CHOMP_FLAGS?) \z//xmso;
+                my $chomp = $1;
+
+                s/\A \Q$tag_end\E //xms or die "Oops!";
+
+                $in_tag = 0;
                 push @tokens, [ code => $code ];
                 if($chomp) {
                     push @tokens, [ postchomp => $chomp ];
@@ -614,8 +627,8 @@ sub define_pair {
     return;
 }
 
-# the low-level tokanizer. Don't use it directly, use advance() instead.
-sub tokanize {
+# the low-level tokenizer. Don't use it directly, use advance() instead.
+sub tokenize {
     my($parser) = @_;
 
     local *_ = \$parser->{input};
@@ -652,7 +665,7 @@ sub next_token_is {
     return $parser->next_token->[1] eq $token;
 }
 
-# the high-level tokanizer
+# the high-level tokenizer
 sub advance {
     my($parser, $expect) = @_;
 
@@ -673,7 +686,7 @@ sub advance {
     $parser->statement_is_finished( $parser->following_newline != 0 );
     my $line = $parser->line( $parser->line + $parser->following_newline );
 
-    $parser->next_token( $parser->tokanize() );
+    $parser->next_token( $parser->tokenize() );
 
     my($arity, $id) = @{$t};
     if( $arity eq "name" && $parser->next_token_is("=>") ) {
